/*
** Module   : UniqueIdGenerator.java
** Abstract : Generator for ids used as UNIQUE-ID attribute values.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 SVL 20130822 Created initial version.
*/
package com.goldencode.p2j.util;

import com.goldencode.p2j.security.*;
import java.util.*;

/**
 * Generator for ids used as UNIQUE-ID attribute values. Generates values that unique across
 * specific kind of entities (temp-tables, buffers, buffer fields etc.) in a specific user
 * context. Note that behavior of this generation may not always match 4GL behavior.
 *
 */
public class UniqueIdGenerator
{
   /**
    * Context-local map of kinds of entities to the next available unique id for this kind.
    */
   private final static ContextLocal<Map<IdKind, integer>> context =
         new ContextLocal<Map<IdKind, integer>>()
         {
            public Map<IdKind, integer> initialValue()
            {
               Map<IdKind, integer> map = new HashMap<IdKind, integer>();
               for (IdKind kind : IdKind.values())
               {
                  map.put(kind, new integer(1));
               }
               return map;
            }
         };

   /**
    * Get new unique id for the specified kind of entities (see {@link IdKind}) in the current
    * user context.
    *
    * @param   kind
    *          Kind of entities for which the id is generated.
    *
    * @return  See above.
    */
   public static integer getUniqueId(IdKind kind)
   {
      Map<IdKind, integer> map = context.get();
      integer id = map.get(kind);
      integer res = new integer(id);

      id.increment();
      return res;
   }

   /**
    * Kind of entities (generates values are unique only across specific kind of entities).
    */
   public enum IdKind
   {
      TEMP_TABLE,
      BUFFER,
      BUFFER_FIELD,
      QUERY
      //TODO etc
   }
}
